<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\ConnectionCustom;
use Teknisa\SM\Util\Utilities;
use Teknisa\SM\Util\Procedure\CriaMnemonico;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;
use Teknisa\SM\Factory\ApiPropostaFactory;
use Teknisa\SM\Factory\Sm00001000LancamentoPropostaFactory;
use Teknisa\SM\Factory\IntegracaoPmFactory;
use Teknisa\SM\Service\SmDocumentosService;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\Remote\cURLRequest;

class ApiPropostaService {
    
    protected $connectionCustom;
    protected $entityManager;
    protected $connection;
    protected $newCode;
    protected $apiPropostaFactory;
    protected $propostaFactory;
    protected $integracaoPmFactory;
    protected $propostaService;
    protected $smDocumentosService;
    
    protected $fileServerBaseUrl;
    protected $apiKeyMapping;
    protected $zhFileUploader;
    
    public function __construct(EntityManager $entityManager, ConnectionCustom $connectionCustom, Sm00001000LancamentoPropostaService $propostaService, $fileServerBaseUrl, $apiKeyMapping) {
        $this->entityManager     = $entityManager;
        $this->connectionCustom  = $connectionCustom;
        $this->propostaService   = $propostaService;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
        $this->apiKeyMapping     = $apiKeyMapping;
    }
    
    private function setValoresPadrao($nrorg) {
        $this->setConnection($nrorg);
        
        $this->newCode = new NewCode($this->connection, "NOVO_CODIGO_SC");
        $this->apiPropostaFactory = new ApiPropostaFactory($this->connection);
        $this->integracaoPmFactory = new IntegracaoPmFactory($this->connection);
        $this->propostaFactory = new Sm00001000LancamentoPropostaFactory($this->entityManager);
        $this->smDocumentosService = new SmDocumentosService($this->apiKeyMapping);
    }
    
    private function setConnection($nrorg) {
        $novoEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
        if ($novoEntityManager) {
            $this->entityManager = $novoEntityManager;
            $this->connection = $novoEntityManager->getConnection();
        }
        else {
            $this->connection = $this->entityManager->getConnection();
        }
    }
    
    public function inserirPropostaApi($cliente, $interlocutor, $proposta, $cdoperador, $nrorg) {
        $this->setValoresPadrao($nrorg);
        $this->connection->beginTransaction();
        try {
            $inscricaoParceiro = str_replace(array(".", "-", "/"), "", $cliente["cpf_cnpj"]);
            //$clienteBD = $this->getClientePorInscricao($inscricaoCliente, $nrorg);
            $parceiroNegocio = $this->getParceiroNegocioPorInscricao($inscricaoParceiro, $nrorg);
            if (is_null($parceiroNegocio)) {
                //inserir parceiro de negocio
                $inscricaoParceiro = str_replace(array(".", "-", "/"), "", $cliente["cpf_cnpj"]);
                $idpessoafisica = "N";
                
                switch (strlen($inscricaoParceiro)) {
                    case 11:
                        $cdtipoinscricao = "CPF";
                        $idtpijurpros = "F";
                        $idpessoafisica = "S";
                        break;
                    case 14:
                        $cdtipoinscricao = "CNPJ";
                        $idtpijurpros = "J";
                        break;
                    default:
                        $cdtipoinscricao = "LIVRE";
                        $idtpijurpros = "I";
                }
                
                //inserir parceiro de negocio
                $nrparcnegocio = $this->newCode->getCodeByTableName("PARCNEGOCIO", $nrorg, 1, 10, true);
                $params = array(
                    "NRPARCNEGOCIO"       => $nrparcnegocio,
                    "NMPRINCIPALPARC"     => $cliente["razao_social"],
                    "NMSECUNDARIPARC"     => $cliente["nome_fantasia"],
                    "NRINSCRICAOPARC"     => $inscricaoParceiro,
                    "CDTIPOPARCPRINCIPAL" => "PROSPECT",
                    "CDTIPOINSCRICAO"     => $cdtipoinscricao,
                    "IDPESSOAFISICA"      => $idpessoafisica,
                    "NRORG"               => $nrorg,
                    "IDATIVO"             => "S",
                    "NRORGINCLUSAO"       => $nrorg,
                    "CDOPERINCLUSAO"      => $cdoperador
                );
                
                $this->apiPropostaFactory->inserirParceiroNegocio($params);
                
                //inserir parceria de prospect
                $nrparceria = $this->newCode->getCodeByTableName("PARCERIA", $nrorg, 1, 10, true);
                $params = array(
                    "NRPARCERIA"     => $nrparceria,
                    "NRPARCNEGOCIO"  => $nrparcnegocio,
                    "CDTIPOPARCEIRO" => "PROSPECT",
                    "NRORG"          => $nrorg,
                    "IDATIVO"        => "S",
                    "NRORGINCLUSAO"  => $nrorg,
                    "CDOPERINCLUSAO" => $cdoperador
                );
                
                $this->apiPropostaFactory->inserirParceria($params);
                
                $cdpais = "0055";
                
                $codigoMunicipio = null;
                $nomeMunicipio = null;
                
                $codigoBairro = null;
                $nomeBairro = $cliente["bairro"];
                
                //busca municipio no banco
                $municipioBD = $this->getMunicipio($cliente["cidade"], $cliente["estado"], $cdpais, $nrorg);
                if ($municipioBD) {
                    $codigoMunicipio = $municipioBD["CDMUNICIPIO"];
                    $nomeMunicipio = $municipioBD["NMMUNICIPIO"];
                    
                    //busca bairro no banco
                    $bairroBD = $this->getBairro($cliente["bairro"], $cliente["cidade"], $cliente["estado"], $cdpais, $nrorg);
                    if ($bairroBD) {
                        $codigoBairro = $bairroBD["CDBAIRRO"];
                        $nomeBairro = $bairroBD["NMBAIRRO"];
                    }
                }
                
                //inserir endereço do parceiro
                $nrenderecoparc = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);
                $params = array(
                    "NRENDERECOPARC"   => $nrenderecoparc,
                    "NRPARCNEGOCIO"    => $nrparcnegocio,
                    "CDTIPOENDERECO"   => "PRINCIPAL",
                    "CDPAIS"           => $cdpais,
                    "SGESTADO"         => $cliente["estado"],
                    "CDMUNICIPIO"      => $codigoMunicipio,
                    "CDBAIRRO"         => $codigoBairro,
                    "NMBAIRROENDERECO" => $nomeBairro,
                    "DSENDERECO"       => $cliente["endereco"],
                    "NRORG"            => $nrorg,
                    "IDATIVO"          => "S",
                    "NRORGINCLUSAO"    => $nrorg,
                    "CDOPERINCLUSAO"   => $cdoperador
                );
                
                $this->apiPropostaFactory->inserirEnderecoParceiroNegocio($params);
                
                
                //TODO
                //inserir comunicacao - email e telefone
                
                
                //inserir prospect
                $cdprospect = $this->newCode->getCodeByTableName("PROSPECT", $nrorg, 1, 10, true);
                $params = array(
                    "CDPROSPECT"     => $cdprospect,
                    "IDTPIJURPROS"   => $idtpijurpros,
                    "NRINSJURPROS"   => $inscricaoParceiro,
                    "NMRAZSOCPROS"   => $cliente["razao_social"],
                    "NMFANTPROS"     => $cliente["nome_fantasia"],
                    "NRINSESTPROS"   => $cliente["inscricao_estadual"],
                    "NRPARCNEGOCIO"  => $nrparcnegocio,
                    "NRORG"          => $nrorg,
                    "IDATIVO"        => "S",
                    "NRORGINCLUSAO"  => $nrorg,
                    "CDOPERINCLUSAO" => $cdoperador
                );
                
                $this->apiPropostaFactory->inserirProspect($params);
                
                //inserir endereço do prospect
                $params = array(
                    "CDPROSPECT"     => $cdprospect,
                    "IDTPENDEPROS"   => "P",
                    "CDPAIS"         => $cdpais,
                    "SGESTADO"       => $cliente["estado"],
                    "CDMUNICIPIO"    => $codigoMunicipio,
                    "CDBAIRRO"       => $codigoBairro,
                    "NMBAIRPROS"     => $nomeBairro,
                    "DSENDEPROS"     => $cliente["endereco"],
                    "NRTELEPROS1"    => $cliente["telefone"],
                    "NRENDERECOPARC" => $nrenderecoparc,
                    "NRORG"          => $nrorg,
                    "IDATIVO"        => "S",
                    "NRORGINCLUSAO"  => $nrorg,
                    "CDOPERINCLUSAO" => $cdoperador
                );
                
                $this->apiPropostaFactory->inserirEnderecoProspect($params);
            }
            else {
                $nrparcnegocio = $parceiroNegocio["NRPARCNEGOCIO"];
            }
            
            $nomeInterlocutor     = $interlocutor["nome"];
            $emailInterlocutor    = $interlocutor["email"];
            $cpfInterlocutor      = $interlocutor["cpf"];
            $telefoneInterlocutor = $interlocutor["telefone"];
            
            $nrseqinterlocutor = $this->getCodigoInterlocutor($emailInterlocutor, $nomeInterlocutor, $nrorg);
            if (is_null($nrseqinterlocutor)) {
                //inserir interlocutor
                $nrseqinterlocutor = $this->newCode->getCodeByTableName("CRM_INTERLOCUTOR", $nrorg, 1, 10, true);
                $params = array(
                    "NRSEQINTERLOCUTOR" => $nrseqinterlocutor,
                    "NRPARCNEGOCIO"     => $nrparcnegocio,
                    "NMINTERLOCUTOR"    => $nomeInterlocutor,
                    "EMAILINTERLOC"     => $emailInterlocutor,
                    "NRCPFINTERLOC"     => $cpfInterlocutor,
                    "NRTELEFONE1"       => $telefoneInterlocutor,
                    "NRORG"             => $nrorg,
                    "IDATIVO"           => "S",
                    "NRORGINCLUSAO"     => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador
                );
                
                $this->apiPropostaFactory->inserirInterlocutor($params);
            }
            
            $nrgeracaoproposta = $proposta["id_layout"];
            
            $layoutGP = $this->getDadosLayoutGeracaoProposta($nrgeracaoproposta, $nrorg);
            $nrorcamentomodelo = $layoutGP["NRORCAMENTOMODELO"];
            $arrNrtipoorcamento = $layoutGP["NRTIPOORCAMENTO"];
            $arrFilialEscopos = $layoutGP["CDFILIALESCOPO"];
            
            $modeloProposta = $this->getDadosModeloProposta($nrorcamentomodelo, $nrorg);
            if (!is_null($modeloProposta)) {
                $nrorcamentomodelo = $modeloProposta["NRORCAMENTOMODELO"];
                $idfaturacontr     = $modeloProposta["IDFATURACONTR"];
                $cdmodpgto         = $modeloProposta["CDMODPGTO"];
                $nrgrupofatura     = $modeloProposta["NRGRUPOFATURA"];
                $qtdiasvalidade    = $modeloProposta["QTDIASVALIDADE"];
                $qtdiascontrato    = $modeloProposta["QTDIASCONTRATO"];
                $dsreferenciaorca  = $modeloProposta["DSREFERENCIAORCA"];
            }
            
            $observacao = null;
            
            $nrseqorcamento = $this->inserirProposta($nrorg, $cdoperador, $dsreferenciaorca, $observacao, $nrseqinterlocutor,
                                                     $emailInterlocutor, $nomeInterlocutor, $nrparcnegocio, $nrorcamentomodelo,
                                                     $idfaturacontr, $cdmodpgto, $nrgrupofatura, $qtdiasvalidade, $qtdiascontrato);
            
            //chama a procedure CRIA_MNEMONICO
            $procedure = new CriaMnemonico($this->connection);
            $procedure->executar($nrorg, "O", $nrseqorcamento);
            
            //inserir escopos
            $escopos = array();
            for ($i = 0; $i < count($arrNrtipoorcamento); $i++) {
                $escopos[] = array(
                    "NRTIPOORCAMENTO" => $arrNrtipoorcamento[$i],
                    "CDFILIAL" => $arrFilialEscopos[$i]
                );
            }
            
            $this->inserirEscoposProposta($nrseqorcamento, $escopos, $nrorg, $cdoperador);
            
            //inserir documentos
            $documentos = $this->getDocumentosEscoposModeloProposta($nrorcamentomodelo, $arrNrtipoorcamento, $nrorg);
            $this->inserirDocumentosProposta($nrseqorcamento, $documentos, $nrorg, $cdoperador);
            
            //inserir composições, elementos e itens da proposta
            $composicoes = $this->getComposicoesEscolhidasLayoutGP($nrgeracaoproposta, $nrorg);
            $elementosEscolhidos = $this->getElementosEscolhidosLayoutGP($nrgeracaoproposta, $nrorg);
            $tiposMedidaVariavel = $this->getTiposMedidaVariavelLayoutGP($nrgeracaoproposta, $nrorg);
            
            $medidasVariaveisIntegracao = $proposta["medidas_variaveis"];
            foreach ($medidasVariaveisIntegracao as $medidaIntegracao) {
                foreach ($tiposMedidaVariavel as &$tipoMedida) {
                    if ($medidaIntegracao["id_elemento"] == $tipoMedida["IDELEMENTORAIZ"] && $medidaIntegracao["id_medida"] == $tipoMedida["CDTPMEDIDASERV"]) {
                        $tipoMedida["QUANTIDADE"] = $medidaIntegracao["qtd_medida"];
                    }
                }
                unset($tipoMedida);
            }
            
            $objProposta = array(
                "NRSEQORCAMENTO" => $nrseqorcamento,
                "NRTIPOORCAMENTO" => $arrNrtipoorcamento
            );
            $itensProposta = $this->propostaService->getItensPropostaParaInserir($objProposta, $elementosEscolhidos, $composicoes, $tiposMedidaVariavel);
            
            $this->propostaService->inserirTudo($nrseqorcamento, $arrNrtipoorcamento, $nrorcamentomodelo, $tiposMedidaVariavel, $itensProposta);
            
            //inserir parcelas
            $negociacoes = $this->getNegociacoesPropostaTipoParcelamento($nrseqorcamento, $nrorg);
            $parcelamento = $proposta["parcelamento"];
            
            foreach ($parcelamento as $parcela) {
                foreach ($negociacoes as &$neg) {
                    if ($parcela["id_escopo"] == $neg["NRTIPOORCAMENTO"]) {
                        $neg["QTPARCELAS"] = $parcela["qtd_parcelas"];
                    }
                }
                unset($neg);
            }
            
            $this->inserirParcelas($nrseqorcamento, $negociacoes, $cdoperador, $nrorg);
            
            //TODO
            //inserir os signatarios da proposta e enviar os documentos para o portal de assinaturas
            
            $this->connection->commit();
            
            return $nrseqorcamento;
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    private function getCodigoInterlocutor($emailinterloc, $nminterlocutor, $nrorg) {
        $params = array(
            "EMAILINTERLOC"  => $emailinterloc,
            "NMINTERLOCUTOR" => $nminterlocutor,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->integracaoPmFactory->getCodigoInterlocutor($params);
        return count($dataset) > 0 ? $dataset[0]["NRSEQINTERLOCUTOR"] : null;
    }
    
    public function getDadosLayoutGeracaoProposta($nrgeracaoproposta, $nrorg) {
        $params = array(
            "NRGERACAOPROPOSTA" => $nrgeracaoproposta,
            "NRORG"             => $nrorg
        );
        
        $dataset = $this->apiPropostaFactory->getDadosLayoutGeracaoProposta($params);
        if (count($dataset) > 0) {
            $layout = $dataset[0];
            $layout["NRTIPOORCAMENTO"] = Utilities::getArrayDeString($layout["NRTIPOORCAMENTO"]);
            $layout["CDFILIALESCOPO"] = Utilities::getArrayDeString($layout["CDFILIALESCOPO"]);
            
            return $layout;
        }
        else {
            throw new \Exception("Não foi encontrado o Layout para Geração de Proposta com o ID '$nrgeracaoproposta'.");
        }
    }
    
    public function getDadosModeloProposta($nrorcamentomodelo, $nrorg) {
        $params = array(
            "NRORCAMENTOMODELO" => $nrorcamentomodelo,
            "NRORG"             => $nrorg
        );
        
        $dataset = $this->apiPropostaFactory->getDadosModeloProposta($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    public function getParceiroNegocioPorInscricao($inscricao, $nrorg) {
        $params = array(
            "NRINSCRICAOPARC" => $inscricao,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->apiPropostaFactory->getParceiroNegocioPorInscricao($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    private function inserirProposta($nrorg, $cdoperador, $dsreferenciaorca, $observacao, $nrseqinterlocutor,
                                     $emailInterlocutor, $nomeInterlocutor, $nrparcnegocio, $nrorcamentomodelo,
                                     $idfaturacontr, $cdmodpgto, $nrgrupofatura, $qtdiasvalidade, $qtdiascontrato) {
        
        $nrseqorcamento = $this->newCode->getCodeByTableName("NRSEQORCAMENTO", $nrorg, 1, 10, true);
        if (!$qtdiasvalidade) $qtdiasvalidade = 30;
        $dataAtual = DateUtil::getDataAtual(true);
        $dataValidade = DateUtil::adicionaIntervalo($dataAtual, $qtdiasvalidade);

        $params = array(
            "NRSEQORCAMENTO"       => $nrseqorcamento,
            "NRPARCNEGOCIO"        => $nrparcnegocio,
            "NRVERSAOORCAMENTO"    => 1,
            "NRPARCTRANSPORTADORA" => null,
            "DTREFORCAMENTO"       => $dataAtual->format("d/m/Y"),
            "CDPRECBASESERV"       => null,
            "CDREDUCONT"           => null,
            "IDFATURAORCA"         => $idfaturacontr,
            "CDMODPGTO"            => $cdmodpgto,
            "NRPARCINTERLOCUTOR"   => null,
            "NRGRUPOFATURA"        => $nrgrupofatura,
            "DSREFERENCIAORCA"     => $dsreferenciaorca,
            "IDTRANSPORTADORA"     => "PROPRIA",
            "QTDIASVALIDADE"       => $qtdiasvalidade,
            "DTVALIDADE"           => $dataValidade->format("d/m/Y"),
            "QTDIASCONTRATO"       => $qtdiascontrato,
            "DSEMAIL"              => $emailInterlocutor,
            "NMPARCINTERLOCUTOR"   => $nomeInterlocutor,
            "NRTELEFONE"           => null,
            "DSOBSERVACAOORCA"     => $observacao,
            "IDGERACONTRATO"       => "NOVO",
            "IDSITUACAOORCAMENTO"  => "CONCLUIDO",
            "VRORCAMENTO"          => 0,
            "NRORG"                => $nrorg,
            "IDATIVO"              => "S",
            "NRORGINCLUSAO"        => $nrorg,
            "CDOPERINCLUSAO"       => $cdoperador,
            "DSINTERORCA"          => $nomeInterlocutor,
            "NRORCAMENTOMODELO"    => $nrorcamentomodelo,
            "DTACEITEORCA"         => null,
            "NRSEQINTERLOCUTOR"    => $nrseqinterlocutor,
            "CDOPERRESPONSAVEL"    => $cdoperador,
            "IDPRODUTO"            => null,
            "ISSUEID"              => null
        );
        
        $this->integracaoPmFactory->inserirProposta($params);
        
        return $nrseqorcamento;
    }
    
    private function inserirEscoposProposta($nrseqorcamento, $escopos, $nrorg, $cdoperador) {
        foreach ($escopos as $escopo) {
            
            $nrrelacescopoproposta = $this->newCode->getCodeByTableName("FCT_RELACESCOPOPROPOSTA", $nrorg, 1, 10, true);
            
            $params = array(
                "NRRELACESCOPOPROPOSTA" => $nrrelacescopoproposta,
                "NRSEQORCAMENTO"        => $nrseqorcamento,
                "NRTIPOORCAMENTO"       => $escopo["NRTIPOORCAMENTO"],
                "NRSEQMODDOCFCT"        => null,
                "CDFILIAL"              => $escopo["CDFILIAL"],
                "NRORG"                 => $nrorg,
                "IDATIVO"               => "S",
                "NRORGINCLUSAO"         => $nrorg,
                "CDOPERINCLUSAO"        => $cdoperador
            );
            
            $this->integracaoPmFactory->inserirEscopoProposta($params);
        }
    }
    
    private function getComposicoesEscolhidasLayoutGP($nrgeracaoproposta, $nrorg) {
        $params = array(
            "NRGERACAOPROPOSTA" => $nrgeracaoproposta,
            "NRORG"             => $nrorg
        );
        
        return $this->apiPropostaFactory->getComposicoesEscolhidasLayoutGP($params);
    }
    
    private function getElementosEscolhidosLayoutGP($nrgeracaoproposta, $nrorg) {
        $params = array(
            "NRGERACAOPROPOSTA" => $nrgeracaoproposta,
            "NRORG"             => $nrorg
        );
        
        $dataset = $this->apiPropostaFactory->getElementosEscolhidosLayoutGP($params);
        return $dataset;
        
        /*$idElementos = array();
        foreach ($dataset as $row) {
            $idElementos[] = $row["IDELEMENTO"];
        }
        
        return $idElementos;*/
    }
    
    private function getTiposMedidaVariavelLayoutGP($nrgeracaoproposta, $nrorg) {
        $params = array(
            "NRGERACAOPROPOSTA" => $nrgeracaoproposta,
            "NRORG"             => $nrorg
        );
        
        return $this->apiPropostaFactory->getTiposMedidaVariavelLayoutGP($params);
    }
    
    private function getNegociacoesPropostaTipoParcelamento($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->propostaFactory->getNegociacoesProposta($params);
        
        return array_filter($dataset, function($row) {
            return $row["IDTPFATURA"] == "P";
        });
    }
    
    private function inserirParcelas($nrseqorcamento, $negociacoes, $cdoperador, $nrorg) {
        foreach ($negociacoes as $neg) {
            $valorNegociacao = $neg["VRTOTITEMCOMDESC"];
            $qtdParcelas = (isset($neg["QTPARCELAS"])) ? $neg["QTPARCELAS"] : 1;
            $qtdMaximaParcelas = $this->getQuantidadeMaximaParcelasEscopoNegociacao($neg["NRNEGOCIACAOPROPOSTA"], $neg["NRTIPOORCAMENTO"], $nrorg);
            
            if (!is_null($qtdMaximaParcelas) && $qtdParcelas > $qtdMaximaParcelas) {
                throw new \Exception("A quantidade de parcelas informada excede a quantidade máxima de parcelas da negociação.<br>Valor máximo permitido: $qtdMaximaParcelas.");
            }
            
            $valorParcela = round($valorNegociacao / $qtdParcelas, 2);
            $somaValorParcelas = 0;
            $qtDiasFaturamento = 30;
            
            for ($i = 1; $i <= $qtdParcelas; $i++) {
            
                if ($i == $qtdParcelas) {
                    $valorParcela = $valorNegociacao - $somaValorParcelas;
                }
                
                $nrparcelanegoprop = $this->newCode->getCodeByTableName("FCT_PARCELANEGOPROP", $nrorg, 1, 10, true);
                $params = array(
                    "NRPARCELANEGOPROP"    => $nrparcelanegoprop,
                    "NRNEGOCIACAOPROPOSTA" => $neg["NRNEGOCIACAOPROPOSTA"],
                    "QTDIASFATURAMENTO"    => $qtDiasFaturamento,
                    "NRDIAVENCIMENTO"      => null,
                    "VRPARCELA"            => $valorParcela,
                    "IDATIVO"              => "S",
                    "NRORG"                => $nrorg,
                    "NRORGINCLUSAO"        => $nrorg,
                    "CDOPERINCLUSAO"       => $cdoperador,
                    "NRPARCELA"            => $i
                );
                
                $this->integracaoPmFactory->inserirParcelaNegociacao($params);
                
                $somaValorParcelas += $valorParcela;
                $qtDiasFaturamento += 30;
            }
        }
    }
    
    private function getQuantidadeMaximaParcelasEscopoNegociacao($nrnegociacaoproposta, $nrtipoorcamento, $nrorg) {
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg,
            "NRTIPOORCAMENTO"      => $nrtipoorcamento,
        );
        $dataset = $this->propostaFactory->getQtMaxParcelasItemEscopo($params);
        
        if ((count($dataset) > 0)) {
            return $dataset[0]["QTPARCELASFATU"];
        }
        else {
            $params = array(
                "NRTIPOORCAMENTO" => $nrtipoorcamento,
                "NRORG"           => $nrorg
            );
            $dataset = $this->propostaFactory->getQtMaxParcelasEscopo($params);
            
            return (count($dataset) > 0) ? $dataset[0]["QTPARCELASFATU"] : null;
        }
    }
    
    private function getDocumentosEscoposModeloProposta($nrorcamentomodelo, $arrNrtipoorcamento, $nrorg) {
        $strNrtipoorcamento = implode($arrNrtipoorcamento, ",");
        $params = array(
            "NRORCAMENTOMODELO" => $nrorcamentomodelo,
            "NRORG"             => $nrorg
        );
        
        return $this->apiPropostaFactory->getDocumentosEscoposModeloProposta($params, $strNrtipoorcamento);
    }
    
    private function inserirDocumentosProposta($nrseqorcamento, $documentos, $nrorg, $cdoperador) {
        $microtime = microtime(true);
        $controleDocsIguais = array();

        foreach ($documentos as $documento) {
            $nome    = $documento["NMARQUIVO"];
            $caminho = $documento["DSCAMINHOARQ"];
            $tipo    = $documento["DSTIPO"];
            $tamanho = $documento["VRTAMANHO"];            

            if (!empty($caminho)) {
                if (array_key_exists($caminho, $controleDocsIguais)) {
                    $nomeNovo    = $controleDocsIguais[$caminho]["NOMENOVO"];
                    $caminhoNovo = $controleDocsIguais[$caminho]["CAMINHONOVO"];
                    $tamanhoNovo = $controleDocsIguais[$caminho]["TAMANHONOVO"];
                    $tipoNovo    = $controleDocsIguais[$caminho]["TIPONOVO"];
                }
                else {
                    $caminhoArq = $this->fileServerBaseUrl . $caminho;
                    
                    $arquivoDoc = $this->smDocumentosService->getDocumentoFileServer($caminhoArq, $nrorg);
                    //$arquivoDoc = Utilities::downloadFileServer($caminhoArq, $this->apiKeyMapping[$nrorg]);
        
                    if ($arquivoDoc !== false) {
                        
                        $this->setZhFileUploader($nrorg);
                        $anexoNovo = Utilities::uploadFileServer($this->zhFileUploader, "anexo_documento_proposta", $nome, $arquivoDoc, $tamanho * 1024, $tipo);
                        
                        $nomeNovo    = $anexoNovo["name"];
                        $caminhoNovo = $anexoNovo["path"];
                        $tamanhoNovo = $anexoNovo["size"];
                        $tamanhoNovo = $tamanhoNovo / 1024;
                        $tipoNovo    = $anexoNovo["type"];

                        $controleDocsIguais[$caminho] = array(
                            "NOMENOVO"    => $nomeNovo,
                            "CAMINHONOVO" => $caminhoNovo,
                            "TAMANHONOVO" => $tamanhoNovo,
                            "TIPONOVO"    => $tipoNovo
                        );
                    }
                    else {
                        $nomeNovo    = null;
                        $caminhoNovo = null;
                        $tamanhoNovo = null;
                        $tipoNovo    = null;
                    }
                }

                if (!is_null($nomeNovo)) {
                    $nrseqorcadoc = $this->newCode->getCodeByTableName("NRSEQORCADOC", $nrorg, 1, 10, true);
                
                    $params = array(
                        "NRSEQORCADOC"        => $nrseqorcadoc,
                        "NRSEQORCAMENTO"      => $nrseqorcamento,
                        "NRSEQMODDOCFCT"      => $documento["NRSEQMODDOCFCT"],
                        "IDIDIOMA"            => $documento["IDIDIOMA"],
                        "TXMODELOORCADOC"     => $documento["TXMODELODOCORCA"],
                        "IDPADRAOORCADOC"     => null,
                        "NRORG"               => $nrorg,
                        "IDATIVO"             => "S",
                        "NRORGINCLUSAO"       => $nrorg,
                        "CDOPERINCLUSAO"      => $cdoperador,
                        "NMARQUIVO"           => $nomeNovo,
                        "DSCAMINHOARQ"        => $caminhoNovo,
                        "DSTIPO"              => $tipoNovo,
                        "VRTAMANHO"           => $tamanhoNovo,
                        "IDSITUACAOORCAMENTO" => $documento["IDSITUACAOORCAMENTO"],
                        "NRTIPOORCAMENTO"     => $documento["NRTIPOORCAMENTO"]
                    );
                    
                    $this->integracaoPmFactory->inserirDocumentosProposta($params);
                }
            }
        }
    }
    
    private function setZhFileUploader($nrorg) {
        if (isset($this->apiKeyMapping[$nrorg])) {
            $key = $this->apiKeyMapping[$nrorg];
            
            $explodeFileServerUrl = explode("/", $this->fileServerBaseUrl);
            
            $fileServerBackendIndexUrl = $explodeFileServerUrl[0] . "//" . $explodeFileServerUrl[2] . "/index.php";
            
            $this->zhFileUploader = new Server(new cURLRequest($fileServerBackendIndexUrl), $key);
        }
        else {
            throw new \Exception("Erro ao anexar documento no FileServer: Não existe chave de API parametrizada para o número de organização informado.");
        }
    }
    
    private function getMunicipio($nmmunicipio, $sgestado, $cdpais, $nrorg) {
        $params = array(
            "NMMUNICIPIO" => $nmmunicipio,
            "SGESTADO"    => $sgestado,
            "CDPAIS"      => $cdpais,
            "NRORG"       => $nrorg
        );
        
        $dataset = $this->apiPropostaFactory->getMunicipio($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    private function getBairro($nmbairro, $cdmunicipio, $sgestado, $cdpais, $nrorg) {
        $params = array(
            "NMBAIRRO"    => $nmbairro,
            "CDMUNICIPIO" => $cdmunicipio,
            "SGESTADO"    => $sgestado,
            "CDPAIS"      => $cdpais,
            "NRORG"       => $nrorg
        );
        
        $dataset = $this->apiPropostaFactory->getBairro($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
}

?>