<?php
namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00003000ParamFatEscopoFactory;

class Sm00003000ParamFatEscopoService {

    protected $paramFatEscopoFactory;
    
    public function __construct(Sm00003000ParamFatEscopoFactory $paramFatEscopoFactory) {
        $this->paramFatEscopoFactory = $paramFatEscopoFactory;
    }
    
    public function getParametrosDeFaturamentoDoEscopo($nrtipoorcamento, $nrorg) {
        $params = array(
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->paramFatEscopoFactory->getParametrosDeFaturamentoDoEscopo($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    public function validarParamsFaturamentoEscopo($nrtipoorcamento, $nrorg) {
        if(is_array($nrtipoorcamento))
            $nrtipoorcamento = implode(",", $nrtipoorcamento);
            
        $params = array( "NRORG" => $nrorg );
        $paramsFaturamento = $this->paramFatEscopoFactory->getEscoposSemParametrosFaturamento($params, $nrtipoorcamento);
        
        foreach($paramsFaturamento as $pfaturamento) {
            if ($pfaturamento["CDFILIALFATURA"] == null)
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Unidade para Faturamento" não foi encontrado para o escopo "' . $pfaturamento["NMTIPOORCAMENTO"] .'". Favor conferir a parametrização de faturamento do Escopo.');
            
            if ($pfaturamento["CDTIPOOPERFATURA"] == null)
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Tipo de Operação da Nota Fiscal" não foi encontrado para o escopo "' . $pfaturamento["NMTIPOORCAMENTO"] .'". Favor conferir a parametrização de faturamento do Escopo.');
            
            if ($pfaturamento["CDSERIENF"] == null)
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Série da Nota Fiscal" não foi encontrado para o escopo "' . $pfaturamento["NMTIPOORCAMENTO"] .'". Favor conferir a parametrização de faturamento do Escopo.');
            
            /*if ($pfaturamento["CDPRODFATURA"] == null)
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Produto para Faturamento" não foi encontrado para o escopo "' . $pfaturamento["NMTIPOORCAMENTO"] .'". Favor conferir a parametrização de faturamento do Escopo ou do Serviço.');
            */
            
            if (in_array($pfaturamento["IDTPFATURA"], array("A", "X"))) {
                $params = array(
                    "NRTIPOORCAMENTO" => $pfaturamento["NRTIPOORCAMENTO"],
                    "NRORG"           => $nrorg
                );
                
                $competencia = $this->paramFatEscopoFactory->getCompetenciaItemNegociacao($params);
                if(count($competencia) == 0)
                    throw new \Exception('Operação bloqueada. <br/><br/>O período de competência não foi encontrado para o escopo "' . $pfaturamento["NMTIPOORCAMENTO"] .'". Favor conferir a parametrização de faturamento do Escopo ou do Serviço.');
            }
        }
    }
    
     /**
     * A função "completaPercentual" verifica qual o percentual faltante para inserção na página "índices Financeiros Padrões"
     */
    
    public function completaPercentual($nrtipoorcamento, $nrorg){
        $params = array(
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "CDINDICE"        => null,
            "NRORG"           => $nrorg
        );
        
        $valida = $this->paramFatEscopoFactory->validaPercentual($params);
        if (!empty($valida[0])){
           $result = 100 - $valida[0]["TOTPERCENT"];
        } else {
           $result =  100; 
        }
        
        return $result;
    }
    
     
     /**
     * A função "validaPercentual" valida o percentual para inserção na página "índices Financeiros Padrões"
     */
    
    public function validaPercentual($nrtipoorcamento, $nrpercentindice, $nrorg, $cdindice = null){
        $params = array(
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg,
            "CDINDICE"        => $cdindice
        );
        
        $valida = $this->paramFatEscopoFactory->validaPercentual($params);
        if (!empty($valida[0])){
           $result = $valida[0]["TOTPERCENT"] + $nrpercentindice;
        } else {
           $result =  $nrpercentindice; 
        }
        
        return $result;
    }
    
    /**
     * Retorna o valor do campo "Tipo de cálculo para parcelas incompletas"
     */
    public function getTipoCalculoPeriodosIncompletos($nrtipoorcamento, $nrorg) {
        $params = array(
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->paramFatEscopoFactory->getTipoCalculoPeriodosIncompletos($params);
        return (count($dataset) > 0) ? $dataset[0]["IDTPCALCPARCINC"] : null;
    }
}

?>